@@ -20,6 +20,8 @@ use crate::vector::{vec2, Vector2D};
 use num_traits::{NumCast, Float};
 #[cfg(feature = "serde")]
 use serde::{Deserialize, Serialize};
+#[cfg(feature = "bytemuck")]
+use bytemuck::{Zeroable, Pod};
 
 use core::borrow::Borrow;
 use core::cmp::PartialOrd;
@@ -99,6 +101,12 @@ impl<T: fmt::Debug, U> fmt::Debug for Box2D<T, U> {
     }
 }
 
+#[cfg(feature = "bytemuck")]
+unsafe impl<T: Zeroable, U> Zeroable for Box2D<T, U> {}
+
+#[cfg(feature = "bytemuck")]
+unsafe impl<T: Pod, U: 'static> Pod for Box2D<T, U> {}
+
 impl<T, U> Box2D<T, U> {
     /// Constructor.
     #[inline]
@@ -108,10 +116,13 @@ impl<T, U> Box2D<T, U> {
 
     /// Constructor.
     #[inline]
-    pub fn from_origin_and_size(origin: Point2D<T, U>, size: Size2D<T, U>) -> Self {
+    pub fn from_origin_and_size(origin: Point2D<T, U>, size: Size2D<T, U>) -> Self 
+    where
+        T: Copy + Add<T, Output = T>
+    {
         Box2D {
             min: origin,
-            max: point2(size.width, size.height),
+            max: point2(origin.x + size.width, origin.y + size.height),
         }
     }
 
@@ -255,6 +266,14 @@ where
         (self.max - self.min).to_size()
     }
 
+    /// Change the size of the box by adjusting the max endpoint
+    /// without modifying the min endpoint.
+    #[inline]
+    pub fn set_size(&mut self, size: Size2D<T, U>) {
+        let diff = (self.size() - size).to_vector();
+        self.max -= diff;
+    }
+
     #[inline]
     pub fn width(&self) -> T {
         self.max.x - self.min.x
@@ -862,4 +881,23 @@ mod tests {
         assert!(Box2D { min: point2(1.0, -2.0), max: point2(NAN, 2.0) }.is_empty());
         assert!(Box2D { min: point2(1.0, -2.0), max: point2(0.0, NAN) }.is_empty());
     }
+
+    #[test]
+    fn test_from_origin_and_size() {
+        let b = Box2D::from_origin_and_size(point2(1.0, 2.0), size2(3.0, 4.0));
+        assert_eq!(b.min, point2(1.0, 2.0));
+        assert_eq!(b.size(), size2(3.0, 4.0));
+    }
+
+    #[test]
+    fn test_set_size() {
+        let mut b = Box2D {
+            min: point2(1.0, 2.0),
+            max: point2(3.0, 4.0),
+        };
+        b.set_size(size2(5.0, 6.0));
+
+        assert_eq!(b.min, point2(1.0, 2.0));
+        assert_eq!(b.size(), size2(5.0, 6.0));
+    }
 }
